from typing import Union, Optional
import base64
import hashlib
from unsafe.exceptions import UnknownEncoding


class Encryptor:
    """
    This is a Class for Text/File Encryption.
    """

    def __init__(self) -> None:
        ...

    def text_encrypt(self, words: str, encode: Optional[str] = None, hash_method: str = 'MD5', **kwargs: str):
        """
        function to return encrypted string.
        
        args:

            * words: your words to hash
            
            `text_encrypt(words = "hello world")`
            
            * encode: your choose to encode(default=None)
            
            `text_encrypt(words = "hello world", encode = "UTF-8")`
            
            * hash_method: your chosen hash method(default=md5)
            
            `text_encrypt(words = "hello world", hash_method = "SHA1")`
            
            `Available Hash Method's : ['MD5', 'SHA1', 'SHA224', 'SHA256', 'SHA384', 'SHA512', 'SHA3-224', 'SHA3-256', 'SHA3-384', 'SHA3-512', 'SHAKE128', 'SHAKE256', 'BASE16', 'BASE32', 'BASE64', 'BASE85', 'ASCII85', 'CAESAR']`

            * count: for 'SHAKE128', 'SHAKE256' and 'CAESAR' method's(default for shake methods = 15 AND default for caesar method = 1)
            
            `text_encrypt(words = "hello world", hash_method = 'SHAKE128', count = 22)`
            
            `text_encrypt(words = "hello world", hash_method = 'CAESAR', count = 4)`
        """

        hash_method = hash_method.lower()

        if isinstance(words, Union[int, float, bool]):
            words = str(words)

        if kwargs != {} and hash_method not in ['shake128', 'shake256', 'caesar']:
            raise ValueError('This Hash Method Have Not Kwargs...')

        if kwargs == {} and hash_method in ['shake128', 'shake256']:
            count = 15
        elif kwargs != {} and hash_method in ['shake128', 'shake256']:
            if 'count' in kwargs:
                try:
                    count = int(kwargs['count'])
                except:
                    raise ValueError('count must be integer !')
            else:
                raise ValueError(
                    f'The argument(s) {[i for i in kwargs.keys()]} not defined !')
        elif kwargs == {} and hash_method == 'caesar':
            count = 1
        elif kwargs != {} and hash_method == 'caesar':
            if 'count' in kwargs:
                try:
                    count = int(kwargs['count'])
                except:
                    raise ValueError('count must be integer !')
            else:
                raise ValueError(
                    f'The argument(s) {[i for i in kwargs.keys()]} not defined !')

        if encode and hash_method == 'caesar' or not encode and hash_method == 'caesar':
            ...
        elif encode:
            try:
                if type(words) == bytes:
                    words = words.decode()
                    words = words.encode()
                else:
                    words = words.encode()
            except LookupError:
                raise UnknownEncoding(encode)
            except Exception as e:
                raise e

        else:
            if type(words) == bytes:
                words = words.decode()
                words = words.encode()
            else:
                words = words.encode()
        if hash_method == 'md5':
            return hashlib.md5(words).hexdigest()
        elif hash_method == 'sha1':
            return hashlib.sha1(words).hexdigest()
        elif hash_method == 'sha256':
            return hashlib.sha256(words).hexdigest()
        elif hash_method == 'sha224':
            return hashlib.sha224(words).hexdigest()
        elif hash_method == 'sha384':
            return hashlib.sha384(words).hexdigest()
        elif hash_method == 'sha512':
            return hashlib.sha512(words).hexdigest()
        elif hash_method == 'sha3-224':
            return hashlib.sha3_224(words).hexdigest()
        elif hash_method == 'sha3-256':
            return hashlib.sha3_256(words).hexdigest()
        elif hash_method == 'sha3-384':
            return hashlib.sha3_384(words).hexdigest()
        elif hash_method == 'sha3-512':
            return hashlib.sha3_512(words).hexdigest()
        elif hash_method == 'base85':
            return base64.b85encode(words)
        elif hash_method == 'base64':
            return base64.b64encode(words)
        elif hash_method == 'base32':
            return base64.b32encode(words)
        elif hash_method == 'base16':
            return base64.b16encode(words)
        elif hash_method == 'ascii85':
            return base64.a85encode(words)
        elif hash_method == 'shake128':
            return hashlib.shake_128(words).hexdigest(count)
        elif hash_method == 'shake256':
            return hashlib.shake_256(words).hexdigest(count)
        elif hash_method == 'caesar':
            if (type(count) != int) or (count not in range(1, 27)):
                raise ValueError(
                    "the key must be an integer between: 1 and 26")
            if (not words) or (len(words) == 0):
                raise ValueError("You must provide data")
            abcd = "abcdefghijklmnopqrstuvwxyz"
            ABCD = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
            hash = ""
            for i in words:
                if str(i) in abcd:
                    hash += abcd[(abcd.index(str(i)) + count) % 26]
                elif str(i) in ABCD:
                    hash += ABCD[(ABCD.index(str(i)) + count) % 26]
                else:
                    hash += str(i)
            return hash
        else:
            raise ValueError('This Hash Method Not Available !')

    def file_encrypt(self, filename: str, encode: Optional[str] = None, hash_method: str = 'MD5',
                     **kwargs: dict[str, int]):
        """
        function to return encrypted string.
        
        args:

            * filename: your filename to hash
            
            `file_encrypt(filename = "path/of/file.text")`
            
            * encode: your choose to encode(default=None)
            
            `file_encrypt(filename = "path/of/file.text", encode = "UTF-8")`
            
            * hash_method: your chosen hash method(default=md5)
            
            `file_encrypt(filename = "path/of/file.text", hash_method = "SHA1")`
            
            `Available Hash Method's : ['MD5', 'SHA1', 'SHA224', 'SHA256', 'SHA384', 'SHA512', 'SHA3-224', 'SHA3-256', 'SHA3-384', 'SHA3-512', 'SHAKE128', 'SHAKE256', 'BASE16', 'BASE32', 'BASE64', 'BASE85', 'ASCII85']`

            * len_count: for 'SHAKE128' and 'SHAKE256' method's(default=15)
            
            `file_encrypt(filename = "path/of/file.text", hash_method = 'SHAKE128, len_count = 22)`
        """
        try:
            with open(filename, "rb") as f:
                words = f.read()
            f.close()

            if hash_method == 'md5':
                return self.text_encrypt(str(words), encode=encode, hash_method='md5', **kwargs)
            elif hash_method == 'sha1':
                return self.text_encrypt(str(words), encode=encode, hash_method='sha1', **kwargs)
            elif hash_method == 'sha256':
                return self.text_encrypt(str(words), encode=encode, hash_method='sha256', **kwargs)
            elif hash_method == 'sha224':
                return self.text_encrypt(str(words), encode=encode, hash_method='sha224', **kwargs)
            elif hash_method == 'sha384':
                return self.text_encrypt(str(words), encode=encode, hash_method='sha384', **kwargs)
            elif hash_method == 'sha512':
                return self.text_encrypt(str(words), encode=encode, hash_method='sha512', **kwargs)
            elif hash_method == 'sha3-224':
                return self.text_encrypt(str(words), encode=encode, hash_method='sha3-224', **kwargs)
            elif hash_method == 'sha3-256':
                return self.text_encrypt(str(words), encode=encode, hash_method='sha3-256', **kwargs)
            elif hash_method == 'sha3-384':
                return self.text_encrypt(str(words), encode=encode, hash_method='sha3-384', **kwargs)
            elif hash_method == 'sha3-512':
                return self.text_encrypt(str(words), encode=encode, hash_method='sha3-512', **kwargs)
            elif hash_method == 'base85':
                return self.text_encrypt(str(words), encode=encode, hash_method='base85', **kwargs)
            elif hash_method == 'base64':
                return self.text_encrypt(str(words), encode=encode, hash_method='base64', **kwargs)
            elif hash_method == 'base32':
                return self.text_encrypt(str(words), encode=encode, hash_method='base32', **kwargs)
            elif hash_method == 'base16':
                return self.text_encrypt(str(words), encode=encode, hash_method='base16', **kwargs)
            elif hash_method == 'ascii85':
                return self.text_encrypt(str(words), encode=encode, hash_method='ascii85', **kwargs)
            elif hash_method == 'shake128':
                return self.text_encrypt(str(words), encode=encode, hash_method='shake128', **kwargs)
            elif hash_method == 'shake256':
                return self.text_encrypt(str(words), encode=encode, hash_method='shake256', **kwargs)
            else:
                raise ValueError('This Hash Method Not Available !')
        except Exception as e:
            raise e



from unsafe.utils.strings import aspx_login, asp_login, php_login, html_login, cgi_login, brf_login, cfm_login, js_login, slash_login, ua, manager, subdomain
import requests
import random
from threading import Thread
from queue import Queue
from socket import gethostbyname
from typing import Optional

class BruteForcer:

    def __init__(self) -> None:
        self.admin_finder_queue = Queue()
        self.file_manager_queue = Queue()
        self.cloudflare_subdomain_bypasser = Queue()
        self.admin_finder_founded = []
        self.file_manager_founded = []
        self.subdomain_bypassed = {}

    def _send_req_admin_finder(self, domain: str, timeout: int, links_queue, user_agent: Optional[str] = None,
                               cookie: Optional[str] = None, proxy: Optional[str] = None, proxies: Optional[list] = None):

        if "http://" in domain:
            domain = domain.replace("http://", "")
        elif "https://" in domain:
            domain = domain.replace("https://", "")
        else:
            ...
        domain = "http://" + domain
        while not links_queue.empty():
            i = links_queue.get()
            try:
                if proxy:
                    proxy = {"http": "http://" +
                             str(proxy), "https": "http://" + str(proxy)}
                elif proxies:
                    random_proxy = random.choice(proxies)
                    proxy = {
                        "http": "http://" + str(random_proxy), "https": "http://" + str(random_proxy)}
                if not user_agent:
                    user_agent = random.choice(ua)
                header = {"User-Agent": user_agent}
                if cookie:
                    header.update({"Cookie": cookie})
                if domain[len(domain) - 1] == "/":
                    domain = domain[0: len(domain) - 1]

                full_link = domain + i
                try:
                    if proxy or proxies:
                        r = requests.get(
                            full_link,
                            headers=header,
                            allow_redirects=False,
                            proxies=proxy,
                            timeout=timeout,
                            verify=False,
                        )
                    else:
                        r = requests.get(
                            full_link,
                            headers=header,
                            allow_redirects=False,
                            timeout=timeout,
                            verify=False,
                        )
                except Exception as e:
                    if len(self.admin_finder_founded) >= 1:
                        return self.admin_finder_founded
                    else:
                        return []
                links_queue.task_done()
                if r.status_code == 200:
                    self.admin_finder_founded.append(full_link)
                else:
                    ...
            except KeyboardInterrupt:
                break

        return self.admin_finder_founded

    def admin_finder(self, domain: str, workers: int = 3, timeout: int = 10, ext: str = "php", user_agent: Optional[str] = None,
                     cookie: Optional[str] = None, proxy: Optional[str] = None, proxies: Optional[list] = None):
        threads = []
        links = []
        self.admin_finder_founded.clear()
        ext = ext.strip().lower()
        if ext == "php":
            links = php_login
        elif ext == "asp":
            links = asp_login
        elif ext == "aspx":
            links = aspx_login
        elif ext == "js":
            links = js_login
        elif ext == "slash":
            links = slash_login
        elif ext == "cfm":
            links = cfm_login
        elif ext == "cgi":
            links = cgi_login
        elif ext == "brf":
            links = brf_login
        elif ext == "html":
            links = html_login
        else:
            raise ValueError("This ext(Argument) Not Allowed !")
        for i in links:
            self.admin_finder_queue.put(i)
        for i in range(workers):
            thread_ = Thread(target=self._send_req_admin_finder, args=(
                domain, timeout, self.admin_finder_queue, user_agent, cookie, proxy, proxies))
            thread_.setDaemon(True)
            thread_.start()
            threads.append(thread_)
        for i in threads:
            i.join()
        return self.admin_finder_founded

    def _send_req_file_manager(self, domain: str, timeout: int, links_queue, user_agent: Optional[str] = None,
                               cookie: Optional[str] = None, proxy: Optional[str] = None, proxies: Optional[list] = None):

        if "http://" in domain:
            domain = domain.replace("http://", "")
        elif "https://" in domain:
            domain = domain.replace("https://", "")
        else:
            ...

        domain = "http://" + domain
        while not links_queue.empty():
            i = links_queue.get()
            try:
                if proxy:
                    proxy = {"http": "http://" +
                             str(proxy), "https": "http://" + str(proxy)}
                elif proxies:
                    random_proxy = random.choice(proxies)
                    proxy = {
                        "http": "http://" + str(random_proxy), "https": "http://" + str(random_proxy)}
                if not user_agent:
                    user_agent = random.choice(ua)
                header = {"User-Agent": user_agent}
                if cookie:
                    header.update({"Cookie": cookie})
                if domain[len(domain) - 1] == "/":
                    domain = domain[0: len(domain) - 1]

                full_link = domain + i
                try:
                    if proxy or proxies:
                        r = requests.get(
                            full_link,
                            headers=header,
                            allow_redirects=True,
                            proxies=proxy,
                            timeout=timeout,
                            verify=True,
                        )
                    else:
                        r = requests.get(
                            full_link,
                            headers=header,
                            allow_redirects=True,
                            timeout=timeout,
                            verify=True,
                        )
                except Exception as e:
                    if len(self.file_manager_founded) >= 1:
                        return self.file_manager_founded
                    else:
                        return []
                links_queue.task_done()
                if r.status_code == 200:
                    self.file_manager_founded.append(f"{r.url}")
                else:
                    ...
            except KeyboardInterrupt:
                break

        return self.file_manager_founded

    def filemanager_finder(self, domain: str, workers: int = 5, timeout: int = 5, user_agent: Optional[str] = None,
                           cookie: Optional[str] = None, proxy: Optional[str] = None, proxies: Optional[list] = None):
        threads = []
        self.file_manager_founded.clear()
        for i in manager:
            self.file_manager_queue.put(i)
        for i in range(workers):
            thread_ = Thread(target=self._send_req_file_manager, args=(
                domain, timeout, self.file_manager_queue, user_agent, cookie, proxy, proxies))
            thread_.setDaemon(True)
            thread_.start()
            threads.append(thread_)
        for i in threads:
            i.join()

        return self.file_manager_founded

    def _send_req_cloudflare_bypasser_subdomain(self, domain: str, links_queue):

        if "http://" in domain:
            domain = domain.replace("http://", "")
        elif "https://" in domain:
            domain = domain.replace("https://", "")
        else:
            ...

        # domain = "http://" + domain
        while not links_queue.empty():
            sub_domain = links_queue.get()
            full_link = f'{sub_domain}.{domain}'
            try:
                ip = gethostbyname(full_link)
                self.subdomain_bypassed.update({full_link: ip})
            except Exception as e:
                if len(self.subdomain_bypassed) >= 1:
                    return self.subdomain_bypassed
                else:
                    return {}
            links_queue.task_done()
        return self.subdomain_bypassed

    def cloudflare_bypasser(self, domain: str, workers: int = 5):
        threads = []
        self.subdomain_bypassed.clear()
        for i in subdomain:
            self.cloudflare_subdomain_bypasser.put(i)
        for i in range(workers):
            thread_ = Thread(target=self._send_req_cloudflare_bypasser_subdomain, args=(
                domain, self.cloudflare_subdomain_bypasser))
            thread_.setDaemon(True)
            thread_.start()
            threads.append(thread_)
        for i in threads:
            i.join()
        return self.subdomain_bypassed



import requests
import bs4
from urllib.parse import urlparse, parse_qs
from typing import Optional
from unsafe.utils.strings import ua
import json, random

class Seeker:
    def __init__(self):
        ...

    def _filter_result(self, link: str):
        try:
            if link.startswith('/url?'):
                o = urlparse(link, 'http')
                link = parse_qs(o.query)['q'][0]
            o = urlparse(link, 'http')
            if o.netloc and 'google' not in o.netloc:
                return link
        except Exception:
            ...

    def google_dorking(self, query: str, timeout: int = 10, proxy: Optional[str] = None):
        res = []
        if proxy:
            r = requests.get(
                f'https://google.com/search?q={query}', timeout=timeout, proxies={"http": proxy, "https": proxy})
        else:
            r = requests.get(
                f'https://google.com/search?q={query}', timeout=timeout)

        soup = bs4.BeautifulSoup(r.text, "html.parser")
        try:
            anchors = soup.find(id='search').findAll('a')
            # Sometimes (depending on the User-agent) there is
            # no id "search" in html response...
        except AttributeError:
            # Remove links of the top bar.
            gbar = soup.find(id='gbar')
            if gbar:
                gbar.clear()
            anchors = soup.findAll('a')
        for i in anchors:
            if self._filter_result(i["href"]):
                res.append(self._filter_result(i["href"]))
            else:
                pass
        return res






from unsafe.utils.encrypt import Encryptor
from unsafe.utils.decrypt import Decrypter
from unsafe.utils.proxy import Proxy
from unsafe.utils.bruteforce import BruteForcer
from unsafe.utils.wp import Wordpress
from unsafe.utils.exifdata import ExifImage
from unsafe.utils.anonymous import Anonymous
from unsafe.utils.seeker import Seeker

class Unsafe(
    Encryptor,
    Decrypter,
    Proxy,
    BruteForcer,
    Wordpress,
    ExifImage,
    Anonymous,
    Seeker
):
    ...



"""IFD user comment tag structure parser module."""

from plum.dump import Record
from plum.str import StrX
from plum.utilities import getbytes

from exif.ifd_tag._base import Base as BaseIfdTag

USER_COMMENT_CHARACTER_CODE_LEN_BYTES = 8


class UserComment(BaseIfdTag):

    """IFD ASCII tag structure parser class."""

    def modify(self, value):
        """Modify tag value.

        :param value: new tag value
        :type value: corresponding Python type

        """
        if (
            len(value) + USER_COMMENT_CHARACTER_CODE_LEN_BYTES
            > self.tag_view.value_count
        ):
            raise ValueError("comment must be no longer than original")

        ifd_tag_str_target = StrX(
            encoding="ascii",
            zero_termination=True,
            nbytes=self.tag_view.value_count - USER_COMMENT_CHARACTER_CODE_LEN_BYTES,
            pad=b"\x00",
            name="ifd_tag_str_target",
        )

        ascii_str_bytes = ifd_tag_str_target.pack(value)
        ascii_replace_start_index = (
            int(self.tag_view.value_offset) + USER_COMMENT_CHARACTER_CODE_LEN_BYTES
        )
        ascii_replace_stop_index = (
            ascii_replace_start_index
            + self.tag_view.value_count
            - USER_COMMENT_CHARACTER_CODE_LEN_BYTES
        )
        self._app1_ref.body_bytes[
            ascii_replace_start_index:ascii_replace_stop_index
        ] = ascii_str_bytes

        self.tag_view.value_count = len(value) + USER_COMMENT_CHARACTER_CODE_LEN_BYTES

    def read(self):
        """Read tag value.

        Since the character code designation itself is 8 bytes, this must be a pointer and cannot exist within an IFD
        tag by itself.

        :returns: tag value
        :rtype: corresponding Python type

        """
        # The string value (not null-terminated) occurs after the character code designation. (All decodable as ASCII.)
        string_value_offset = (
            int(self.tag_view.value_offset) + USER_COMMENT_CHARACTER_CODE_LEN_BYTES
        )
        string_len = (
            int(self.tag_view.value_count) - USER_COMMENT_CHARACTER_CODE_LEN_BYTES
        )

        value_bytes, _ = getbytes(
            buffer=self._app1_ref.body_bytes,
            offset=string_value_offset,
            dump=Record(),
            nbytes=string_len,
        )
        return StrX(encoding="ascii", name="ascii_str").unpack(value_bytes)

    def set_character_code_to_ascii(self):
        """Set the character code header to ASCII."""
        header_replace_start_index = int(self.tag_view.value_offset)
        header_replace_stop_index = (
            int(self.tag_view.value_offset) + USER_COMMENT_CHARACTER_CODE_LEN_BYTES
        )
        self._app1_ref.body_bytes[
            header_replace_start_index:header_replace_stop_index
        ] = b"ASCII\x00\x00\x00"



"""Legacy Windows XP style tag structure parser module."""

from plum.dump import Record
from plum.utilities import getbytes

from exif.ifd_tag._base import Base as BaseIfdTag


class WindowsXp(BaseIfdTag):

    """Legacy Windows XP style tag structure parser class."""

    def modify(self, value):  # pragma: no cover
        """Modify tag value.

        :param value: new tag value
        :type value: corresponding Python type

        """
        raise NotImplementedError(
            "this package does not yet support setting Windows XP style tags"
        )

    def read(self):
        """Read tag value.

        In string types, the count refers to how many characters exist in the string. Windows XP tags are encoded in
        UCS2/UTF-16, not ASCII.

        :returns: tag value
        :rtype: corresponding Python type

        """
        dereferenced_bytes, _ = getbytes(
            buffer=self._app1_ref.body_bytes,
            offset=int(self.tag_view.value_offset),
            dump=Record(),
            nbytes=int(self.tag_view.value_count),
        )

        return dereferenced_bytes.decode("utf-16")[:-1]  # discard null terminator



from PIL import Image
from pathlib import Path
import os
from exif import Image


class ExifImage:
    def __init__(self) -> None:
        ...

    def extract_exif_img(self, path: Path):
        if not os.path.isfile(path):
            raise ValueError("Your entered path not exists or not image.")
        with open(path, 'rb') as image_file:
            image = Image(image_file)
        if image.has_exif == True:
            exif_list = image.list_all()
            exif_dict = {}
            for i in exif_list:
                try:
                    exif_dict[f"{i}"] = f"{image[i]}"
                except:
                    ...
            return exif_dict
        else:
            return {}

    def delete_exif_img(self, path: Path):
        if not os.path.isfile(path):
            raise ValueError("Your entered path not exists or not image.")
        try:
            with open(path, 'rb') as image_file:
                image = Image(image_file)

            image.delete_all()

            with open(path, 'wb') as new_image_file:
                new_image_file.write(image.get_file())
            return True
        except:
            return False

    def edit_exif_image(self, path: Path, key: str, value: str):
        if not os.path.isfile(path):
            raise ValueError("Your entered path not exists or not image.")
        with open(path, 'rb') as image_file:
            image = image_file.read()
        try:
            image = Image(image)
            image[f"{key}"] = value
            with open(path, 'wb') as new:
                new.write(image.get_file())
            return True
        except Exception as e:
            return False
